/** adc_dma.ino 
 *  
 * A program egy analóg - digitális konverter (ADC) adatainak gyűjtésére szolgál 
 * DMA segítségével. A minta mérete 512, az ADC bemeneti csatorna 0, a 
 * mintavételi frekvencia 1000 Hz. 
 * Az adatok a Serial virtuális soros porton keresztül kerülnek kiírásra. 
 * 
 * Hardver követelmények:
 * - RAspberry Pi Pico, vagy más RP2040 kártya
 * - Analóg bemenő jel a GPIO26 (ADC0) bemeneten
 * 
 * Tested with Arduino IDE 1.8.19 and Arduino MbedOS RP2040 Boards v4.2.1  
 */
#include <mbed.h>
#include <hardware/adc.h>
#include <hardware/dma.h>
#include <hardware/irq.h>

#define BUFFER_SIZE 512
#define ADC_CHANNEL 0
#define SAMPLE_RATE 1000

uint16_t adcBuffer[BUFFER_SIZE];
int dma_channel;
volatile bool bufferFull = false;

void dma_handler() {
  bufferFull = true;
  dma_hw->ints0 = 1u << dma_channel;  // Töröljük a DMA megszakítás jelzést
}

void setup() {
  Serial.begin(115200);

  // ADC inicializálása
  adc_init();
  adc_gpio_init(26 + ADC_CHANNEL);  // GPIO 26, ha ADC_CHANNEL 0
  adc_select_input(ADC_CHANNEL);

  // FIFO beállítása
  adc_fifo_setup(
    true,    // FIFO engedélyezése
    true,    // DMA adat kimenet engedélyezése
    1,       // FIFO küszöbérték (1 elem esetén DMA kérés)
    false,   // Error flag 15. bitben
    true     // Adat jobbra shiftelés
  );

  // DMA inicializálása
  dma_channel = dma_claim_unused_channel(true);
  dma_channel_config cfg = dma_channel_get_default_config(dma_channel);
  channel_config_set_transfer_data_size(&cfg, DMA_SIZE_16);
  channel_config_set_read_increment(&cfg, false);
  channel_config_set_write_increment(&cfg, true);
  channel_config_set_dreq(&cfg, DREQ_ADC);

  dma_channel_configure(
    dma_channel,
    &cfg,
    adcBuffer,          // Cél cím (adcBuffer)
    &adc_hw->fifo,      // Forrás cím (ADC FIFO)
    BUFFER_SIZE,        // Átmásolandó adatok száma
    true                // Azonnal indítsa a DMA-t
  );

  // DMA megszakítás beállítása
  dma_channel_set_irq0_enabled(dma_channel, true);
  irq_set_exclusive_handler(DMA_IRQ_0, dma_handler);
  irq_set_enabled(DMA_IRQ_0, true);

  // ADC mintavételezési sebesség beállítása
  adc_set_clkdiv(48000000 / (SAMPLE_RATE * 96) - 1);  // Helyes órajel osztó
  adc_run(true);  // ADC indítása
}

void loop() {
  if (bufferFull) {
    bufferFull = false;
    // Adatok kilistázása
    for (int i = 0; i < BUFFER_SIZE; i++) {
      Serial.println(adcBuffer[i]);
    }
    // DMA újraindítása
    dma_channel_set_write_addr(dma_channel, adcBuffer, false);
    dma_channel_set_trans_count(dma_channel, BUFFER_SIZE, true);
  }
}
