/**
   Dining Philosophers Problem

   This program demonstrates the classical synchronization problem
   known as the "Dining Philosophers Problem."
   The problem was designed to illustrate issues related to concurrency,
   resource sharing, and deadlock prevention in a multi-threaded environment.

   In this program, there are five philosophers sitting around a table.
   Each philosopher alternates between thinking and eating.
   There are five chopsticks (shared resources) on the table, one between
   each pair of adjacent philosophers. A philosopher needs both chopsticks to eat.

   The challenge is to ensure that no two adjacent philosophers eat simultaneously,
   to avoid a deadlock situation where no philosopher can eat.

   Mutexes are used to solve this problem by ensuring that each philosopher can lock
   and unlock the chopsticks in a safe and controlled manner. This prevents conflicts
   and ensures that only one philosopher can access the chopsticks at a time.

   Tested using Arduino IDE v1.8.19 and the Arduino MbedOS RP2040 board environment.
*/

#include "mbed.h"
#include "rtos.h"

using namespace mbed;
using namespace rtos;

Mutex chopstick[5];  // Array of mutexes representing the 5 chopsticks

void philosopher(const void *args) {
  int num = *(static_cast<const int*>(args));         // Typecast of args
  while (true) {
    if (chopstick[num - 1].trylock()) {
      if (chopstick[num % 5].trylock()) {
        printf("Philosopher %d is EATING\n", num);    // Start EATING
        ThisThread::sleep_for(1000 + rand() % 1000);
        chopstick[num % 5].unlock();  // Release chopsticks
        chopstick[num - 1].unlock();
        printf("Philosopher %d is THINKING\n", num);  // Start THINKING
        ThisThread::sleep_for(2000 + rand() % 2000);  // Gets hungry after this time...
      } else {
        chopstick[num - 1].unlock();
        ThisThread::sleep_for(100 + rand() % 100);    // Wait for random time if failed
      }
    } else {
      ThisThread::sleep_for(100 + rand() % 100);      // Wait for random time if failed
    }
  }
}

int main() {
  int task_ids[5] = {1, 2, 3, 4, 5};
  Thread threads[4];
  // Start four threads
  for (int i = 0; i < 4; i++) {
    threads[i].start(callback(philosopher, &task_ids[i]));
  }

  // Run the fifth philosopher in the main thread
  philosopher(&task_ids[4]);
}
