/** dual_core.ino
 *  
 * Here we calculate the approximate value of Pi using the weakly 
 * converging Gregory-Leibniz series on two cores of the 
 * Raspberry Pi Pico. The number of iterations (1 million) is 
 * halved between the two cores. The program measures the execution 
 * time and displays the results via the serial port.
 * 
 * The program demonstrates significant performance improvement 
 * by distributing the task between two cores.
 * 
 * Tested with:
 * - Arduino IDE v1.8.19
 * - Arduino Mbed OS RP2040 Core v4.2.1
 */

 The program measures the execution time and displays the results via the serial port.
#include <Arduino.h>
#include "pico/multicore.h"

volatile double result0 = 0.0;
volatile double result1 = 0.0;
volatile int ledstate = LOW;
volatile  unsigned long n = 1000000; // Iterációk száma
volatile unsigned long n_2 = n / 2;

void core1_task() {
  while (true) {
    int flip = multicore_fifo_pop_blocking();
    // Billentjük a GPIO14 kimenetet
    digitalWrite(14, flip);
    // Hosszú számolás (Gregory-Leibniz sorozat)
    volatile double pi = 0.0;
    for (unsigned long i = 0; i < n_2; ++i) {
      if (i % 2 == 0) {
        pi += 1.0 / (2 * i + 1);
      } else {
        pi -= 1.0 / (2 * i + 1);
      }
    }
    result1 = pi * 4;
    multicore_fifo_push_blocking(flip);
  }
}

void core0_task() {
  // Hosszú számolás (Gregory-Leibniz sorozat)
  volatile double pi = 0.0;
  for (unsigned long i = n_2; i < n; ++i) {
    if (i % 2 == 0) {
      pi += 1.0 / (2 * i + 1);
    } else {
      pi -= 1.0 / (2 * i + 1);
    }
  }
  result0 = pi * 4;
}

void setup() {
  Serial.begin(115200);
  pinMode(14, OUTPUT);
  delay(1000);
  // Elindítjuk a második magon futó feladatot
  multicore_launch_core1(core1_task);
}

void loop() {
  Serial.println("Start...");
  // Kezdjük el mérni az időt
  unsigned long start_time = millis();
  multicore_fifo_push_blocking(ledstate);
  core0_task();
  int flop = multicore_fifo_pop_blocking();
  // Időmérés befejezése
  unsigned long end_time = millis();
  unsigned long execution_time = end_time - start_time;
  ledstate = !ledstate;
  // Kiírjuk a futási időt
  Serial.print("Futási idő két maggal: ");
  Serial.print(execution_time);
  Serial.println(" ms");
  
  // Kiírjuk az eredményeket (csak ellenőrzésképpen)
  Serial.print("Eredmény az első magról: ");
  Serial.println(result0, 10); // 10 tizedesjegyig pontos
  Serial.print("Eredmény a második magról: ");
  Serial.println(result1, 10); // 10 tizedesjegyig pontos

  // Összesített eredmény
  double total_pi = result0 + result1; // Az eredmények összeadása
  Serial.print("Összesített pi érték: ");
  Serial.println(total_pi, 10); // 10 tizedesjegyig pontos
  delay(2000);
}
