/**
 * @file RedirectedSerial.cpp
 * @brief A library to redirect `printf()` output to the Arduino Serial (USB) port.
 * 
 * This library allows you to use the standard `printf()` function to print messages
 * to the Arduino Serial (USB) port instead of the default Mbed OS UART (Serial1).
 * It is particularly useful for debugging and logging purposes on the Raspberry Pi Pico
 * when using the Arduino Mbed OS RP2040 core.
 * 
 * @author Istvan Cserny
 * @date 2025-02-14
 * @version 1.0.0
 * @license MIT
 * 
 * @note This library is designed for use with the Arduino Mbed OS RP2040 core.
 * 
 * @example BasicUsage/BasicUsage.ino
 * This example demonstrates how to use the `RedirectedSerial` library to redirect
 * `printf()` output to the Serial (USB) port.
 *
 * Tested with Arduino IDE 1.8.19 and Arduino Mbed OS RP2040 core v4.2.1 
 */
#include "RedirectedSerial.h"

// RedirectedSerial osztály metódusai
ssize_t RedirectedSerial::write(const void *buffer, size_t size) {
    return Serial.write((const uint8_t*)buffer, size);
}

ssize_t RedirectedSerial::read(void *buffer, size_t size) {
    return Serial.readBytes((char*)buffer, size);
}

int RedirectedSerial::close() {
    return 0;
}

off_t RedirectedSerial::seek(off_t offset, int whence) {
    return -ESPIPE;
}

off_t RedirectedSerial::size() {
    return -1;
}

int RedirectedSerial::isatty() {
    return true;
}

int RedirectedSerial::sync() {
    return 0;
}

short RedirectedSerial::poll(short events) const {
    return events;
}

// mbed_override_console függvény implementációja
RedirectedSerial mySerial;

namespace mbed {
    FileHandle *mbed_override_console(int fd) {
        return &mySerial;
    }
}