/**
 * @file RedirectedSerial.h
 * @brief A library to redirect `printf()` output to the Arduino Serial (USB) port.
 * 
 * This library allows you to use the standard `printf()` function to print messages
 * to the Arduino Serial (USB) port instead of the default Mbed OS UART (Serial1).
 * It is particularly useful for debugging and logging purposes on the Raspberry Pi Pico
 * when using the Arduino Mbed OS RP2040 core.
 * 
 * @author Istvan Cserny
 * @date 2025-02-14
 * @version 1.0.0
 * @license MIT
 * 
 * @note This library is designed for use with the Arduino Mbed OS RP2040 core.
 * 
 * @example BasicUsage/BasicUsage.ino
 * This example demonstrates how to use the `RedirectedSerial` library to redirect
 * `printf()` output to the Serial (USB) port.
 *
 * Tested with Arduino IDE 1.8.19 and Arduino Mbed OS RP2040 core v4.2.1 
 */

#ifndef REDIRECTEDSERIAL_H
#define REDIRECTEDSERIAL_H

#include "mbed.h"
#include <Arduino.h>

/**
 * @class RedirectedSerial
 * @brief A class that overrides the Mbed OS `FileHandle` to redirect output to Serial (USB).
 * 
 * This class implements the `mbed::FileHandle` interface and overrides its methods
 * to redirect all `printf()` output to the Arduino Serial (USB) port.
 */
class RedirectedSerial : public mbed::FileHandle {
public:
    /**
     * @brief Write data to the Serial (USB) port.
     * 
     * @param buffer The data to write.
     * @param size The number of bytes to write.
     * @return The number of bytes written.
     */
    virtual ssize_t write(const void *buffer, size_t size);

    /**
     * @brief Read data from the Serial (USB) port.
     * 
     * @param buffer The buffer to store the read data.
     * @param size The number of bytes to read.
     * @return The number of bytes read.
     */
    virtual ssize_t read(void *buffer, size_t size);

    /**
     * @brief Close the file handle (not used in this implementation).
     * 
     * @return Always returns 0 (success).
     */
    virtual int close();

    /**
     * @brief Seek to a specific position (not supported for Serial).
     * 
     * @param offset The offset to seek to.
     * @param whence The reference point for the offset.
     * @return Always returns -ESPIPE (illegal seek).
     */
    virtual off_t seek(off_t offset, int whence = SEEK_SET);

    /**
     * @brief Get the size of the file (not supported for Serial).
     * 
     * @return Always returns -1 (unsupported).
     */
    virtual off_t size();

    /**
     * @brief Check if the file handle is a terminal.
     * 
     * @return Always returns 1 (true).
     */
    virtual int isatty();

    /**
     * @brief Synchronize the file (not used in this implementation).
     * 
     * @return Always returns 0 (success).
     */
    virtual int sync();

    /**
     * @brief Poll the file handle for events.
     * 
     * @param events The events to poll for.
     * @return The events that are ready.
     */
    virtual short poll(short events) const;
};

/**
 * @brief Override the Mbed OS console to use the `RedirectedSerial` class.
 * 
 * This function is called by Mbed OS to redirect the console output to the
 * `RedirectedSerial` class, which sends the output to the Serial (USB) port.
 * 
 * @param fd The file descriptor (not used in this implementation).
 * @return A pointer to the `RedirectedSerial` object.
 */
namespace mbed {
    FileHandle *mbed_override_console(int fd);
}

#endif // REDIRECTEDSERIAL_H